const NS_RDF_NO_VALUE			= 0xa0002;
const NS_RDF_ASSERTION_REJECTED = 0xa0003;

var RDF =
  Components
  .classes["@mozilla.org/rdf/rdf-service;1"]
  .getService(Components.interfaces.nsIRDFService);

const NC = "http://home.netscape.com/NC-rdf#";
const NC_NAME		  = RDF.GetResource(NC + "Name");
const NC_MANGLED_NAME = RDF.GetResource(NC + "mangled-name");

function calendarDataSource()
{
  this.mInner = RDF.GetDataSource("rdf:files");
  this.mInner.AddObserver(this);
  this.mObservers = new Array();
}

calendarDataSource.prototype = {
  /* nsIRDFDataSource */
  URI: "rdf:calendar",

  GetSource: function(pred, obj, tv) {
	/* XXX ought to handle NS_MANGLED_NAME here. */
	return this.mInner.GetSource(pred, obj, tv);
  },

  GetSources: function(pred, obj, tv) {
	/* XXX ought to handle NS_MANGLED_NAME here. */
	return this.mInner.GetSources(pred, obj, tv);
  },

  GetTarget: function(subj, pred, tv) {
	if (pred == NC_MANGLED_NAME) {
	  var name = this.mInner.GetTarget(subj, NC_NAME, true)
		.QueryInterface(Components.interfaces.nsIRDFLiteral)
		.Value;

	  return RDF.GetLiteral(rot13(name));
	}
	return this.mInner.GetTarget(subj, pred, tv);
  },

  GetTargets: function(subj, pred, tv) {
	/* XXX ought to handle NS_MANGLED_NAME here. */
	return this.mInner.GetTargets(subj, pred, tv);
  },

  Assert: function(subj, pred, obj, tv) 	   { throw NS_RDF_ASSERTION_REJECTED; },
  Unassert: function(subj, pred, obj, tv)	   { throw NS_RDF_ASSERTION_REJECTED; },
  Change: function(subj, pred, oldobj, newobj) { throw NS_RDF_ASSERTION_REJECTED; },
  Move: function(oldsubj, newsubj, pred, obj)  { throw NS_RDF_ASSERTION_REJECTED; },

  HasAssertion: function(subj, pred, obj, tv) {
	if (pred == NC_MANGLED_NAME) {
	  var unmangled = obj
		.QueryInterface(Components.interfaces.nsIRDFLiteral);

	   if (obj)
		return this.mInner.HasAssertion(subj, pred,
										RDF.GetLiteral(rot13(unmangled.Value)),
										tv);
	}
	return this.mInner.HasAssertion(subj, pred, obj, tv);
  },

  AddObserver: function(aObserver) {
	this.mObservers.push(aObserver);
  },

  RemoveObserver: function(aObserver) {
	for (var i = 0; i < this.mObservers.length; ++i) {
	  if (aObserver == this.mObservers[i]) {
		this.mObservers.splice(i, 1);
		break;
	  }
	}
  },

  ArcLabelsIn: function(obj) {
	/* XXX may need to append NC_MANGLED_NAME here. */
	return this.mInner.ArcLabelsIn(obj);
  },

  ArcLabelsOut: function(subj) {
	/* XXX may need to append NC_MANGLED_NAME here. */
	return this.mInner.ArcLabelsOut(obj);
  },

  GetAllResources: function() {
	return this.mInner.GetAllResources();
  },

  hasArcIn: function(obj, pred) {
	/* XXX may need to check NC_MANGLED_NAME here. */
	return this.mInner.hasArcIn(obj, pred);
  },

  hasArcOut: function(subj, pred) {
	/* XXX may need to check NC_MANGLED_NAME here. */
	return this.mInner.hasArcOut(subj, pred);
  },

  /* nsIRDFObserver */
  onAssert: function(ds, subj, pred, obj) {
	/* XXX may need to check changes to NC_NAME here, and
	   additionally broadcast change to NC_MANGLED_NAME. */
	for (var obs in this.mObservers)
	  obs.onAssert(this, subj, pred, obj);
  },

  onUnassert: function(ds, subj, pred, obj) {
	/* XXX may need to check changes to NC_NAME here, and
	   additionally broadcast change to NC_MANGLED_NAME. */
	for (var obs in this.mObservers)
	  obs.onUnassert(this, subj, pred, obj);
  },

  onChange: function(ds, subj, pred, oldobj, newobj) {
	/* XXX may need to check changes to NC_NAME here, and
	   additionally broadcast change to NC_MANGLED_NAME. */
	for (var obs in this.mObservers)
	  obs.onChange(ds, subj, pred, oldobj, newobj);
  },

  onMove: function(ds, oldsubj, newsubj, pred, obj) {
	/* XXX may need to check changes to NC_NAME here, and
	   additionally broadcast change to NC_MANGLED_NAME. */
	for (var obs in this.mObservers)
	  obs.onMove(ds, oldsubj, newsubj, pred, obj);
	}
  };

  var DS = new MangledFilesDataSource;

  function init()
  {
	var outliner = document.getElementById("outliner");
	outliner.database.AddDataSource(DS);
	outliner.setAttribute("ref", "NC:FilesRoot");
  }

function finish()
{
  // XXX better way to do this? We need to break the circularity
  // between DS and DS.mInner.
  DS.mInner.RemoveObserver(DS);
}

